
package edu.columbia.logging.file;

import java.io.File;
import java.net.URL;
import java.util.Date;

public interface IChatTranscript {
	public static final int TYPE_RICH_TEXT = 1;    // Valid saveToFile type
	public static final int TYPE_PLAIN_TEXT = 2;   // Valid saveToFile type
    
    /**
     * Get the date of this log entry
     * @return
     */
    public Date getDate();
    
    /**
     * Get the date/time of last activity
     * @return
     * @deprecated - Should not be used and eventually will be removed
     */
    public Date getLastActivity();
    
    /**
     * Get the initiator of this chat message
     * @deprecated - Should not be used and eventually will be removed
     */
    public String getChatInitiator();
    
    /**
     * @return String[] recipient with one entry non-null for a one-on-one chat and null for a multi-person chat
     * @deprecated - Should not be used and eventually will be removed
     */
    public String[] getChatRecipients();
    
    /**
     * Return the interface regarding the individual this transcript is from
     * @return
     */
    public IChatPersonHistory getChatPersonHistory();
    
    /** 
     * Get the topic of this chat message
     * @return String the topic (may be empty)
     */
    public String getChatTopic();
    
    /** 
     * Get the contents this chat message
     * @return the contents (typically HTML text) including header and footer
     */
    public String getContents();
    
    /** 
     * Get the contents this chat message without header/footer
     * @return the contents (typically HTML text) excluding header and footer
     * NOTE: This is typically for use for pasting into something which also contains 
     * the header and footer
     */
    public String getContentsWithoutHeaderAndFooter();
    
    /** 
     * Get URL pointing to the contents of the message if can be so represented. 
     * @return the URL or <code>null</code>if contents cannot be represented by URL
     * @deprecated - Should not be used and eventually will be removed
     */
    public URL getContentsURL();
    
    /** 
     * Get File containing the contents of the message. 
     * @return the File or <code>null</code>
     */
    public File getContentsFile();
    
    /** 
     * Get Strings representing full elements of meta information
     * @return String[] separate strings containing complete META elements
     */
    public String[] getMetaInfo();

	/**
	 * Answer whether the chat is n-way
	 * @return <code>true</code> if n-way, <code>false</code> otherwise
	 */
    public boolean isChatNWay();
    
    /**
     * Save the contents of the transcript to the given file with the given type
     * @param saveFile file to save to
     * @param saveAsType type plain or rich
     * @return <code>true</code> if successful, <code>false</code> otherwise
     */
    public boolean saveToFile(File saveFile, int saveAsType);
    
     /**
     * Save to default location
     * @return <code>true</code> if successful, <code>false</code> otherwise
     */
    public boolean save();
    
    /**
     * Delete this transcript
     * @return <code>true</code> if successful
     */
    public boolean delete();
    
    /**
     * Close this transcript.  Typically used for before closing actions or cleanup.
     * @return <code>true</code> if successful
     */
    public boolean close();
    
    /**
     * Set the header text
     */
    public void setHeader(String header);
    
    /**
     * Set the footer
     */
    public void setFooter(String footer);
    
    /**
     * Append text
     */
    public void append(String contents);
    
    /**
     * Send the transcript via email
     * @param to The person(s) to receive the email
     * @param subject The subject for the email
     * @param preTranscriptHTML Information to be shown before the transcript
     */
    public void sendAsEmail(String to, String subject, String preTranscriptHTML);
}
